import "package:flutter/material.dart";
import 'dart:math' as math;
import 'Icon_picker_untils.dart';
import 'flutter_icons_data.dart';

class FlutterIconPicker extends StatefulWidget {
  final bool randomIconColors;
  final Color? color;
  final ValueChanged onChanged;
  final double iconSize;
  final int iconsPerRow;

  const FlutterIconPicker({
    Key? key,
    this.randomIconColors = false,
    required this.onChanged,
    this.color,
    this.iconSize = 33,
    this.iconsPerRow = 4,
  }) : super(key: key);

  @override
  State<FlutterIconPicker> createState() => _FlutterIconPickerState();
}

class _FlutterIconPickerState extends State<FlutterIconPicker> {
  List randomColorsList = [];
  List iconsList = [];

  @override
  void initState() {
    initialize();

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final cmh = MediaQuery.of(context).size.height;

    final cmw = MediaQuery.of(context).size.width;
    return SizedBox(
      height: cmh * 0.49,
      width: cmw * 0.67,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: [
          ///here is the search bar
          Container(
            decoration: BoxDecoration(
                color: Color(0xfff8f9fa),
                border: Border.all(
                  color: Color(0xff707070).withOpacity(.45),
                ),
                borderRadius: BorderRadius.circular(cmh * 0.1)),
            child: TextFormField(
              textAlign: TextAlign.center,
              decoration: InputDecoration(
                  isDense: true,
                  contentPadding: EdgeInsets.only(
                    left: cmw * 0.015,
                    right: cmw * 0.015,
                    top: cmw * 0.021,
                    bottom: cmw * 0.021,
                  ),
                  border: InputBorder.none,
                  hintText: 'search icon...',
                  hintStyle: TextStyle(
                      color: const Color(0xff707070).withOpacity(.75))),
              onChanged: (value) {
                setState(() =>
                    iconsList = IconPickerUtils().searchIcon(query: value));
              },
            ),
          ),
          SizedBox(
            height: cmh * 0.023,
          ),

          ///here are the icons displayed
          Expanded(
            child: Scrollbar(
              interactive: true,
              child: GridView.builder(
                  itemCount: iconsList.length,
                  // itemCount: testIconData.length,
                  gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      crossAxisCount: widget.iconsPerRow),
                  itemBuilder: (context, index) {
                    List iconList = iconsList;

                    ///here are all the icons displayed
                    return IconButton(
                      splashRadius: 25,
                      icon: Icon(
                        IconData(iconList[index]['icon_code'],
                            fontFamily: 'MaterialIcons'),

                        ///icon Size
                        size: widget.iconSize,

                        ///icon color
                        color: widget.color ??
                            (widget.randomIconColors == false &&
                                    widget.color == null
                                ? const Color(0xff2196f3)
                                : randomColorsList[index]),
                      ),
                      onPressed: () {
                        Navigator.of(context).pop();

                        ///here is the data returned
                        var iconName = iconsList[index]['icon_name'];
                        var iconCode = iconsList[index]['icon_code'];
                        var flutterIcon = iconsList[index]['flutter_icon'];
                        var color = widget.color ??
                            (widget.randomIconColors == false &&
                                    widget.color == null
                                ? const Color(0xff2196f3)
                                : randomColorsList[index]);

                        widget.onChanged({
                          'icon_name': iconName,
                          'icon_code': iconCode,
                          'flutter_icon': flutterIcon,
                          'color': color,
                        });
                      },
                    );
                  }),
            ),
          ),
        ],
      ),
    );
  }

  ///this is the initialize function
  initialize() {
    setState(() {
      iconsList = flutterIconsData;
    });

    ///this generated Random Colors List (a random color for all the available icons)
    generateRandomColors();
  }

  ///this is the generate function
  ///generates a random color for each icon
  generateRandomColors() {
    for (var i = 0; i < flutterIconsData.length; i++) {
      setState(() {
        randomColorsList.add(
            Color((math.Random().nextDouble() * 0xFFFFFF).toInt())
                .withOpacity(1.0));
      });
    }
  }
}
